﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using www.openworkbpm.com.schema;

namespace TestSendEvent
{
    class Program
    {
        // Indirizzo di base (da modificare solo su specifiche indicazioni)
        static string realBaseAddress = "https://paas.jamio.com/gateway/gateway/";

        // parametri e credenziali da impostare
        static string userName = "";
        static string password = "";
        static string identityRepositoryName = "";
        static string repositoryId = "";
        static string eventTestModelUrl = "";
        static string eventTestDataSetModelUrl = "";
        static string eventTestArraysModelUrl = "";
        static string eventTestReferenceModelUrl = "";
        // fine parametri

        static void Main(string[] args)
        {

            // rappresenta un identificativo del sistema che effettua la richiesta
            // a discrezione dello sviluppatore
            string source = Environment.MachineName;
            //creazione del client per la gestione delle sessioni
            SessionManagementClient SessionManagement = new SessionManagementClient("basicHttpsBinding_Gateway_ISessionManagement", realBaseAddress + "ISessionManagement");

            //Invocazione del servizio di login
            ServiceResultOfLoginInfo loginResult = SessionManagement.Login(identityRepositoryName, userName, password, false, source);
            if (!loginResult.Error)
            {
                Console.WriteLine("Login: OK");
            }
            else
            {
                Console.WriteLine("Login: KO. " + loginResult.ResultInfoList[0].Message);
                SessionManagement.Close();
                return;
            }

            //creazione dell’oggetto richiesto dalle invocazioni contente il token di sessione
            RequestInfo info = new RequestInfo();
            info.Token = loginResult.Value.Token;

            // invio dell'evento semplice
            SendTestEvent(createTestEvent(), info);

            // invio dell'esempio con dataset
            SendTestEvent(createTestEventDataSet(), info, new List<Type> {typeof(DataSetValue)});

            // invio dell'esempio con array
            SendTestEvent(createTestEventArrays(), info, new List<Type> { typeof(ArrayValue) });

            // invio dell'esempio con reference
            SendTestEvent(createTestEventReference(), info, new List<Type> { typeof(NamedReference),typeof(CatalogItemValue) });

            // logout e chiusura sessione
            Console.WriteLine("Logout...");
            SessionManagement.CloseSession(info);
            SessionManagement.Close();
            Console.WriteLine("Invio eventi terminato... Premi INVIO per uscire");
            Console.Read();

        }

        private static void SendTestEvent(CustomEvent EventToSend, RequestInfo info, List<Type> KnownTypes = null)
        {
            //creazione del client per la gestione degli eventi
            EventManagementClient EventManagement = new EventManagementClient("basicHttpsBinding_Gateway_IEventManagement", realBaseAddress + "IEventManagement");
            if (KnownTypes!= null)
                foreach (Type KnownType in KnownTypes)
                    addKnownType(EventManagement.Endpoint, KnownType);

            // invio dell'evento
            ServiceResultOfboolean eventResult = EventManagement.PublishEvent(repositoryId, EventToSend, info);
            if (!eventResult.Error)
            {
                Console.WriteLine("Publish " + EventToSend.ModelReference.Name + ": OK");
            }
            else
            {
                Console.WriteLine("Publish " + EventToSend.ModelReference.Name + ": KO. " + eventResult.ResultInfoList[0].Message);
            }
            EventManagement.Close();
        }

        private static void addKnownType(System.ServiceModel.Description.ServiceEndpoint proxyEndPoint, Type knownType)
        {
            foreach (var op in proxyEndPoint.Contract.Operations)
            {
                if (!op.KnownTypes.Contains(knownType))
                    op.KnownTypes.Add(knownType);
            }
        }

        private static CustomEvent createTestEvent()
        {
            var EventToSend = new CustomEvent();

            //modello dell'evento
            EventToSend.ModelReference = new NamedReference
            {
                Name = "TestEvent",
                Type = Types.CustomEvent,
                Url = eventTestModelUrl
            };

            EventToSend.Fields = new Fields();

            EventToSend.Fields.Add("campo stringa", "campo stringa");
            EventToSend.Fields.Add("campo numerico", 1);
            EventToSend.Fields.Add("campo booleano", true);
            EventToSend.Fields.Add("campo data", DateTime.Now);

            return EventToSend;
        }

        private static CustomEvent createTestEventDataSet()
        {
            var EventToSend = new CustomEvent();

            //modello dell'evento
            EventToSend.ModelReference = new NamedReference
            {
                Name = "TestEventDataSet",
                Type = Types.CustomEvent,
                Url = eventTestDataSetModelUrl
            };

            EventToSend.Fields = new Fields();

            EventToSend.Fields.Add("campo stringa", "campo stringa");
            EventToSend.Fields.Add("campo numerico", 1);
            EventToSend.Fields.Add("campo booleano", true);
            EventToSend.Fields.Add("campo data", DateTime.Now);

            DataSetValue dataset = new DataSetValue();
            dataset.Fields = new Fields();
            dataset.Fields.Add("booleano", true);
            dataset.Fields.Add("float", 1.2);
            dataset.Fields.Add("dataeora", DateTime.Now);
            dataset.Fields.Add("testo", "nunzio");
            dataset.Fields.Add("decimale", Convert.ToDecimal(1));
            dataset.Fields.Add("intero", 1);

            EventToSend.Fields.Add("campo dataset", dataset);
            return EventToSend;
        }

        private static CustomEvent createTestEventArrays()
        {
            var EventToSend = new CustomEvent();

            //modello dell'evento
            EventToSend.ModelReference = new NamedReference
            {
                Name = "TestEventArrays",
                Type = Types.CustomEvent,
                Url = eventTestArraysModelUrl
            };

            EventToSend.Fields = new Fields();


            EventToSend.Fields.Add("campo stringa", new ArrayValue { Elements = new SerializableListOfanyType() { "nunzio", "nunzio1" } });
            EventToSend.Fields.Add("campo numerico", new ArrayValue { Elements = new SerializableListOfanyType() { 1 } });
            EventToSend.Fields.Add("campo booleano", new ArrayValue { Elements = new SerializableListOfanyType() { true } });
            EventToSend.Fields.Add("campo data", new ArrayValue { Elements = new SerializableListOfanyType() { DateTime.Now } });

            DataSetValue dataset = new DataSetValue();
            dataset.Fields = new Fields();
            dataset.Fields.Add("booleano", true);
            dataset.Fields.Add("float", 1.2);
            dataset.Fields.Add("dataeora", DateTime.Now);
            dataset.Fields.Add("testo", "nunzio");
            dataset.Fields.Add("decimale", Convert.ToDecimal(1));
            dataset.Fields.Add("intero", 1);
            EventToSend.Fields.Add("campo dataset", new ArrayValue { Elements = new SerializableListOfanyType() { dataset } });
            return EventToSend;
        }

        private static CustomEvent createTestEventReference()
        {
            var EventToSend = new CustomEvent();

            //modello dell'evento
            EventToSend.ModelReference = new NamedReference
            {
                Name = "TestEventReference",
                Type = Types.CustomEvent,
                Url = eventTestReferenceModelUrl
            };

            EventToSend.Fields = new Fields();

            EventToSend.Fields.Add("campo collegamento", new NamedReference { Name = "nunzio", Type = Types.Account, Url = "url" });

            CatalogItemValue extref = new CatalogItemValue();
            extref.Fields = new Fields();
            extref.Fields.Add("booleano", true);
            extref.ModelReference = new NamedReference { Name = "name", Type = Types.AccountCriteria, Url = "url" };
            extref.Name = "name";
            extref.Type = Types.AccountCriteriaEvent;
            extref.Url = "url";

            EventToSend.Fields.Add("campo esteso", extref);
            return EventToSend;
        }
    }
}
